/*
    liban - base function to do usefull things related to simulation and scientific work
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (c) 2004-2005 Antoine Beyeler <antoine.beyeler@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software. You can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __AN_NUMERIC_H
#define __AN_NUMERIC_H

/*!	\file Numeric.h
	\brief Some mathematic classes for numeric operations
*/

namespace An
{
	/**
	Implements an exponential decay using the Crank-Nicholson method. For improved
	performances, the CN factor is cached, assuming that the dt will rarely change
	during runtime.
	*/
	class ExpDecay
	{
	public:
		//! Constructor
		//! @param t Time-constant
		//! @param delta Integration step
		//! @param init Initial value
		ExpDecay(double t = 5.0, double delta = 1.0, double init = 0.0)
		{
			tau = t;
			dt = delta;
			value = init;
			syncFactor();
		}
	
		// Operations.
		//! Add f to actual value
		void add(double f) { value += f; }
		//! Decay during the same time as last step
		void step() { value *= factor; }
		//! Decay during a given time
		//! @p thisDt decay time
		void step(double thisDt) { setDt(thisDt); step(); }
		//! Return the value and than decay for the same time as last step
		double getValueAndStep() { double tmp = getValue(); step(); return tmp; }
		//! Return the value and than decay during a given time
		//! @p thisDt decay time
		double getValueAndStep(double thisDt) { setDt(thisDt); return getValueAndStep(); }
		
		// Accessors.
		//! Set the time constant
		//! @p t time constant
		void setTau(double t) { if (t != tau) { tau = t; syncFactor(); } }
		//! Return the time constant
		double getTau() const { return tau; }
		//! Set the step duration
		//! @p delta step duration
		void setDt(double delta) { if (delta != dt) { dt = delta; syncFactor(); } }
		//! Return the step duration
		double getDt() const { return dt; }
		//! Set the value
		//! @p val value
		void setValue(double val) { value = val; }
		//! Return the value
		double getValue() const { return value; }
		
		// Operators.
		//! Set the value
		//! @p val value
		void operator = (double val) { setValue(val); }
		//! Add f to the value
		void operator += (double f) { add(f); }
		//! Substract f from the value
		void operator -= (double f) { add(-f); }
		//! Return the value
		operator double() const { return getValue(); }
		
	protected:
		double tau; //!< time constant
		double factor; //!< factor for Crank-Nicholson method, computed from tau and value
		double value; //!< actual value
		double dt; //!< step duration
		
		//! Calc step multiplication factor according to the Crank-Nicholson method.
		void syncFactor() { factor = (tau - dt * 0.5) / (tau + dt * 0.5); }
	};
}

#endif
